@@ -2,11 +2,9 @@ from __future__ import unicode_literals
 
 from collections import defaultdict
 
-from django.contrib.contenttypes.models import ContentType
 from django.contrib import messages
 from django.utils.translation import ugettext as _
-from django.contrib.admin.models import LogEntry, CHANGE
-from django.utils.encoding import force_text, force_unicode
+from django.utils.encoding import force_text
 from django.contrib.admin.templatetags.admin_urls import add_preserved_filters
 from django.http import HttpResponseRedirect
 
@@ -39,7 +37,10 @@ class FSMTransitionMixin(object):
     * In the absence of specific transition permissions, the user must
       have change permission for the model.
     """
-
+    # Each transition input is named with the transition.
+    # e.g. _fsmtransition-publish
+    #      _fsmtransition-delete
+    fsm_input_prefix = '_fsmtransition'
     # name of the FSMField on the model to transition
     fsm_field = 'state'
     change_form_template = 'fsm_admin/change_form.html'
@@ -51,7 +52,7 @@ class FSMTransitionMixin(object):
         Available state transitions are provided by django-fsm
         following the pattern get_available_FIELD_transitions
         """
-        transitions_func = 'get_available_{}_transitions'.format(self.fsm_field)
+        transitions_func = 'get_available_{0}_transitions'.format(self.fsm_field)
         transitions = getattr(obj, transitions_func)() if obj else []
         return transitions
 
@@ -83,48 +84,15 @@ class FSMTransitionMixin(object):
         Override of `ModelAdmin.response_change` to detect the FSM button
         that was clicked in the submit row and perform the state transtion.
         """
-        # Each transition button is named with the transition.
-        # e.g. _fsmtransition-publish
-        #      _fsmtransition-delete
-        transition_key = [k for k in request.POST.keys() if k.startswith("_fsmtransition")]
-        if not transition_key:
+        if not getattr(obj, '_fsmtransition_results', None):
             return super(FSMTransitionMixin, self).response_change(request, obj)
 
-        # Extract the function name from the transition key
-        transition = transition_key[0].split('-')[1]
-        original_state = self.display_fsm_field(obj)
-        msg_dict = {
-            'obj': force_text(obj),
-            'transition': transition,
-        }
-
-        # Ensure the requested transition is availble
-        transitions = self._fsm_get_transitions(obj)
-        available = any([t.name == transition for t in transitions])
-        trans_func = getattr(obj, transition, None)
-
-        if available and trans_func:
-            # Run the transition
-            try:
-                #Attempt to pass in the by argument if using django-fsm-log
-                trans_func(by=request.user)
-            except TypeError:
-                #If the function does not have a by attribute, just call with no arguments
-                trans_func()
-
-            # The transition may not be marked to automatically save, so
-            # we assume that it should always be saved.
-            obj.save()
-            new_state = self.display_fsm_field(obj)
-
-            # Done! Log the change and message user
-            self.log_state_change(obj, request.user.id, original_state, new_state)
-            msg_dict.update({'new_state': new_state})
-            msg = _('%(obj)s successfully set to %(new_state)s') % msg_dict
-            self.message_user(request, msg, messages.SUCCESS)
+        if obj._fsmtransition_results['status'] == messages.SUCCESS:
+            msg = _('%(obj)s successfully set to %(new_state)s') % obj._fsmtransition_results
         else:
-            msg = _('Error! %(obj)s failed to %(transition)s') % msg_dict
-            self.message_user(request, msg, messages.ERROR)
+            msg = _('Error! %(obj)s failed to %(transition)s') % obj._fsmtransition_results
+
+        self.message_user(request, msg, obj._fsmtransition_results['status'])
 
         opts = self.model._meta
         redirect_url = self.get_redirect_url(request=request, obj=obj)
@@ -133,18 +101,53 @@ class FSMTransitionMixin(object):
         redirect_url = add_preserved_filters({'preserved_filters': preserved_filters, 'opts': opts}, redirect_url)
         return HttpResponseRedirect(redirect_url)
 
-    def log_state_change(self, obj, user_id, original_state, new_state):
+    def _transition_is_available(self, obj, transition):
         """
-        Log the transition of the object to the history.
+        Check if the requested transition is available
         """
-        LogEntry.objects.log_action(
-            user_id=user_id,
-            content_type_id=ContentType.objects.get_for_model(obj.__class__).pk,
-            object_id=obj.pk,
-            object_repr=force_unicode(obj),
-            action_flag=CHANGE,
-            change_message='Changed {} from {} to {}'.format(self.fsm_field_instance.verbose_name, original_state, new_state),
-        )
+        transitions = self._fsm_get_transitions(obj)
+        return any([t.name == transition for t in transitions])
+
+    def _get_transition_keys(self, request):
+        return [k for k in request.POST.keys() if k.startswith(self.fsm_input_prefix)]
+
+    def _transition(self, transition_keys, request, obj, form):
+        transition = transition_keys[0].split('-')[1]
+        original_state = self.display_fsm_field(obj)
+        msg_dict = {
+            'obj': force_text(obj),
+            'transition': transition,
+            'original_state': original_state,
+        }
+        # Ensure the requested transition is available
+        available = self._transition_is_available(obj, transition)
+        trans_func = getattr(obj, transition, None)
+        if available and trans_func:
+            # Run the transition
+            try:
+                #Attempt to pass in the by argument if using django-fsm-log
+                trans_func(by=request.user)
+            except TypeError:
+                #If the function does not have a by attribute, just call with no arguments
+                trans_func()
+            new_state = self.display_fsm_field(obj)
+
+            # Mark the fsm_field as changed in the form so it will be
+            # picked up when the change message is constructed
+            form.changed_data.append(self.fsm_field)
+
+            msg_dict.update({'new_state': new_state, 'status': messages.SUCCESS})
+        else:
+            msg_dict.update({'status': messages.ERROR})
+
+        # Attach the results of our transition attempt
+        setattr(obj, '_fsmtransition_results', msg_dict)
+
+    def save_model(self, request, obj, form, change):
+        transition_keys = self._get_transition_keys(request)
+        if transition_keys:
+            self._transition(transition_keys, request, obj, form)
+        obj.save()
 
     def get_transition_hints(self, obj):
         """
